<?php
class StorageProtos {
    protected static $internal_static_textsecure_SessionStructure_descriptor;    // com.google.protobuf.Descriptors.Descriptor
    protected static $internal_static_textsecure_SessionStructure_fieldAccessorTable;    // com.google.protobuf.GeneratedMessage.FieldAccessorTable
    protected static $internal_static_textsecure_SessionStructure_Chain_descriptor;    // com.google.protobuf.Descriptors.Descriptor
    protected static $internal_static_textsecure_SessionStructure_Chain_fieldAccessorTable;    // com.google.protobuf.GeneratedMessage.FieldAccessorTable
    protected static $internal_static_textsecure_SessionStructure_Chain_ChainKey_descriptor;    // com.google.protobuf.Descriptors.Descriptor
    protected static $internal_static_textsecure_SessionStructure_Chain_ChainKey_fieldAccessorTable;    // com.google.protobuf.GeneratedMessage.FieldAccessorTable
    protected static $internal_static_textsecure_SessionStructure_Chain_MessageKey_descriptor;    // com.google.protobuf.Descriptors.Descriptor
    protected static $internal_static_textsecure_SessionStructure_Chain_MessageKey_fieldAccessorTable;    // com.google.protobuf.GeneratedMessage.FieldAccessorTable
    protected static $internal_static_textsecure_SessionStructure_PendingKeyExchange_descriptor;    // com.google.protobuf.Descriptors.Descriptor
    protected static $internal_static_textsecure_SessionStructure_PendingKeyExchange_fieldAccessorTable;    // com.google.protobuf.GeneratedMessage.FieldAccessorTable
    protected static $internal_static_textsecure_SessionStructure_PendingPreKey_descriptor;    // com.google.protobuf.Descriptors.Descriptor
    protected static $internal_static_textsecure_SessionStructure_PendingPreKey_fieldAccessorTable;    // com.google.protobuf.GeneratedMessage.FieldAccessorTable
    protected static $internal_static_textsecure_RecordStructure_descriptor;    // com.google.protobuf.Descriptors.Descriptor
    protected static $internal_static_textsecure_RecordStructure_fieldAccessorTable;    // com.google.protobuf.GeneratedMessage.FieldAccessorTable
    protected static $internal_static_textsecure_PreKeyRecordStructure_descriptor;    // com.google.protobuf.Descriptors.Descriptor
    protected static $internal_static_textsecure_PreKeyRecordStructure_fieldAccessorTable;    // com.google.protobuf.GeneratedMessage.FieldAccessorTable
    protected static $internal_static_textsecure_SignedPreKeyRecordStructure_descriptor;    // com.google.protobuf.Descriptors.Descriptor
    protected static $internal_static_textsecure_SignedPreKeyRecordStructure_fieldAccessorTable;    // com.google.protobuf.GeneratedMessage.FieldAccessorTable
    protected static $internal_static_textsecure_IdentityKeyPairStructure_descriptor;    // com.google.protobuf.Descriptors.Descriptor
    protected static $internal_static_textsecure_IdentityKeyPairStructure_fieldAccessorTable;    // com.google.protobuf.GeneratedMessage.FieldAccessorTable
    protected static $internal_static_textsecure_SenderKeyStateStructure_descriptor;    // com.google.protobuf.Descriptors.Descriptor
    protected static $internal_static_textsecure_SenderKeyStateStructure_fieldAccessorTable;    // com.google.protobuf.GeneratedMessage.FieldAccessorTable
    protected static $internal_static_textsecure_SenderKeyStateStructure_SenderChainKey_descriptor;    // com.google.protobuf.Descriptors.Descriptor
    protected static $internal_static_textsecure_SenderKeyStateStructure_SenderChainKey_fieldAccessorTable;    // com.google.protobuf.GeneratedMessage.FieldAccessorTable
    protected static $internal_static_textsecure_SenderKeyStateStructure_SenderMessageKey_descriptor;    // com.google.protobuf.Descriptors.Descriptor
    protected static $internal_static_textsecure_SenderKeyStateStructure_SenderMessageKey_fieldAccessorTable;    // com.google.protobuf.GeneratedMessage.FieldAccessorTable
    protected static $internal_static_textsecure_SenderKeyStateStructure_SenderSigningKey_descriptor;    // com.google.protobuf.Descriptors.Descriptor
    protected static $internal_static_textsecure_SenderKeyStateStructure_SenderSigningKey_fieldAccessorTable;    // com.google.protobuf.GeneratedMessage.FieldAccessorTable
    protected static $internal_static_textsecure_SenderKeyRecordStructure_descriptor;    // com.google.protobuf.Descriptors.Descriptor
    protected static $internal_static_textsecure_SenderKeyRecordStructure_fieldAccessorTable;    // com.google.protobuf.GeneratedMessage.FieldAccessorTable
    protected static $descriptor;    // com.google.protobuf.Descriptors.FileDescriptor
    private function __init() { // default class members
    }
    public static function __staticinit() { // static class members
    }
    public static function constructor__ ()
    {
        $me = new self();
        $me->__init();
        return $me;
    }
    abstract static function registerAllExtensions ($registry); // [com.google.protobuf.ExtensionRegistry registry]
    public static function getDescriptor ()
    {
        return self::$descriptor;
    }
}
StorageProtos::__staticinit(); // initialize static vars for this class on load
